/*----------------------------------------------------------*
    dxlib.h
    Dynamixelプロトコル用API V4.2
                                       Last Edit '19 03/19
   Copyright (c) 2005, 2019 BestTechnology CO.,LTD.
 *----------------------------------------------------------*/

#ifndef _DXLIB_H_INCLUDE
#define _DXLIB_H_INCLUDE

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _WIN32
  #include <windows.h>
#else
  #include <pthread.h>
#endif

#ifndef _MSC_VER
  #include <stdbool.h>
#endif

#include <stdint.h>

#define BROADCASTING_ID     (0xfe)

#define DXMAXLENGTH         (250)

#ifndef _WIN32
  #ifdef __MAKE_LIB__
    typedef int                             HANDLE;
    typedef pthread_mutex_t                 CRITICAL_SECTION;

    #define EnterCriticalSection(mtx)       pthread_mutex_lock(mtx)
    #define LeaveCriticalSection(mtx)       pthread_mutex_unlock(mtx)
    #define DeleteCriticalSection(mtx)      pthread_mutex_destroy(mtx)
    #define InitializeCriticalSection(mtx)  pthread_mutex_init(mtx,NULL)
    #define SetEvent(evt)                   pthread_cond_signal(evt)
    #define INVALID_int_VALUE               -1
    #define INVALID_HANDLE_VALUE            -1
  #endif
#endif

// Device ID
#ifdef _WIN32
  #ifdef _WIN64
    typedef uint64_t TDeviceID;
  #else
    typedef uint32_t TDeviceID;
  #endif
  #define _PACKED_  __attribute__((gcc_struct,__packed__))
#else
  #if defined(__x86_64__) | defined(__aarch64__ )
    typedef uint64_t TDeviceID;
    #if defined(__x86_64__)
      #define _PACKED_  __attribute__((gcc_struct,__packed__))
    #else
      #define _PACKED_  __attribute__((__packed__))
    #endif
  #else
    typedef uint32_t TDeviceID;
    #if defined(__x86__)
      #define _PACKED_  __attribute__((gcc_struct,__packed__))
    #else
      #define _PACKED_  __attribute__((__packed__))
    #endif
  #endif
#endif

// Kind of instruction
typedef enum {INST_PING = 1, INST_READ = 2, INST_WRITE = 3, INST_REG_WRITE = 4, INST_ACTION = 5, INST_RESET = 6, INST_SYNC_WRITE = 0x83, INST_SYNG_REG_WRITE = 0x85} TInstruction;

// Error flags
typedef uint16_t TErrorCode;
#define ERR_INVALID_DEVID     (1 << 15)
#define ERR_INVALID_ID        (1 << 14)
#define ERR_DIFF_ID           (1 << 13)
#define ERR_ILLEGAL_SIZE      (1 << 12)
#define ERR_INVALID_PARAM     (1 << 11)
#define ERR_COMM              (1 << 10)
#define ERR_CHECKSUM          (1 << 9)
#define ERR_TIMEOUT           (1 << 8)
#define ERR_DX_INST           (1 << 6)
#define ERR_DX_OVERLOAD       (1 << 5)
#define ERR_DX_CHECKSUM       (1 << 4)
#define ERR_DX_RANGE          (1 << 3)
#define ERR_DX_OVERHEAT       (1 << 2)
#define ERR_DX_ANGLE          (1 << 1)
#define ERR_DX_OVERVOLTAGE    (1 << 0)
#define ERR_NON               (0)

// Alarm status
#ifdef _MSC_VER
  __pragma (pack (push, 1))
  typedef struct {
    uint8_t     id;
    TErrorCode  Status;
  } TDxAlarmStatus;

  typedef struct {
    uint8_t addr;
    uint8_t length;
    uint8_t ids[256];
  } TSyncReadParam;
  __pragma (pack (pop))
#elif defined(__GNUC__)
  typedef struct {
    uint8_t     id;
    TErrorCode  Status;
  } _PACKED_ TDxAlarmStatus;

  typedef struct {
    uint8_t addr;
    uint8_t length;
    uint8_t ids[256];
  } _PACKED_ TSyncReadParam;
#endif

// デバイスの種別
typedef enum {
  devtNONE, devtDX, devtAX, devtRX, devtEX, devtMX, devtXL320, devtPRO, devtX
} TDXL_DevType;

// 全シリーズモデル情報
#ifdef _MSC_VER
  __pragma (pack (push, 1))
  typedef struct {
    uint16_t  modelno;          // デバイス固有のモデル番号
    char      name[16];
    TDXL_DevType devtype;       // デバイスのグループ
    struct {                    // 位置の範囲
      int32_t max;
      int32_t min;
    }         positionlimit;
    struct {                    // 角度の範囲
      double  max;
      double  min;
    }         anglelimit;
    struct {                    // 速度の範囲
      int32_t max;
      int32_t min;
    }         velocitylimit;
    struct {                    // PWMの範囲
      int32_t max;
      int32_t min;
    }         pwmlimit;
    double    velocityratio;    // 角速度変換係数 [deg/sec]
    double    currentratio;     // 電流変換係数 [mA]
    double    pwmratio;         // PWM変換係数 [%]
  } TDXL_ModelInfo, *PDXL_ModelInfo;
  __pragma (pack (pop))
#elif defined(__GNUC__)
  typedef struct {
    uint16_t  modelno;          // デバイス固有のモデル番号
    char      name[16];
    TDXL_DevType devtype;       // デバイスのグループ
    struct {                    // 位置の範囲
      int32_t max;
      int32_t min;
    } _PACKED_ positionlimit;
    struct {                    // 角度の範囲
      double  max;
      double  min;
    } _PACKED_ anglelimit;
    struct {                    // 速度の範囲
      int32_t max;
      int32_t min;
    } _PACKED_ velocitylimit;
    struct {                    // PWMの範囲
      int32_t max;
      int32_t min;
    } _PACKED_ pwmlimit;
    double    velocityratio;    // 角速度変換係数 [deg/sec]
    double    currentratio;     // 電流変換係数 [mA]
    double    pwmratio;         // PWM変換係数 [%]
  } _PACKED_ TDXL_ModelInfo, *PDXL_ModelInfo;
#endif

//
typedef struct {
  double  angle;
  double  velocity;
} TAngleVelocity, *PAngleVelocity;

typedef struct {
  double  angle;
  double  sec;
} TAngleTime, *PAngleTime;

#ifdef _WIN32
  #ifdef __cplusplus
    #ifdef __MAKE_LIB__
      #define DXAPIDLL extern "C" __declspec(dllexport)
    #else
      #define DXAPIDLL extern "C" __declspec(dllimport)
    #endif
  #else
    #ifdef __MAKE_LIB__
      #define DXAPIDLL __declspec(dllexport)
    #else
      #define DXAPIDLL __declspec(dllimport)
    #endif
  #endif
#else
  #define DXAPIDLL
  #define WINAPI
#endif

#ifndef _DYNAMICLOAD
// When static DLL calls is used the following prototype declaration
DXAPIDLL TDeviceID WINAPI DX_OpenPort (char *name, uint32_t baud);
DXAPIDLL bool WINAPI DX_ClosePort (TDeviceID dvid);
DXAPIDLL bool WINAPI DX_SetBaudrate (TDeviceID dvid, uint32_t baud);
DXAPIDLL bool WINAPI DX_Active (TDeviceID dvid);
DXAPIDLL void WINAPI DX_SetTimeOutOffset (TDeviceID dvid, uint32_t t);
DXAPIDLL double WINAPI GetQueryPerformanceCounter (void);
DXAPIDLL bool WINAPI DX_TxPacket (TDeviceID dvid, uint8_t id, TInstruction inst, const uint8_t *param, uint32_t len, TErrorCode *err);
DXAPIDLL bool WINAPI DX_RxPacket (TDeviceID dvid, uint8_t *rdata, uint32_t rdatasize, uint32_t *rlen, uint32_t timeout, TErrorCode *err);
DXAPIDLL bool WINAPI DX_ReadByteData (TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *rdata, TErrorCode *err);
DXAPIDLL bool WINAPI DX_WriteByteData (TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t dat, TErrorCode *err);
DXAPIDLL bool WINAPI DX_ReadWordData (TDeviceID dvid, uint8_t id, uint16_t adr, uint16_t *rdata, TErrorCode *err);
DXAPIDLL bool WINAPI DX_WriteWordData (TDeviceID dvid, uint8_t id, uint16_t adr, uint16_t dat, TErrorCode *err);
DXAPIDLL bool WINAPI DX_ReadLongData (TDeviceID dvid, uint8_t id, uint16_t adr, uint32_t *rdata, TErrorCode *err);
DXAPIDLL bool WINAPI DX_WriteLongData (TDeviceID dvid, uint8_t id, uint16_t adr, uint32_t dat, TErrorCode *err);
DXAPIDLL bool WINAPI DX_ReadBlockData (TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *rdata, uint32_t len, TErrorCode *err);
DXAPIDLL bool WINAPI DX_WriteBlockData (TDeviceID dvid, uint8_t id, uint16_t adr, const uint8_t *dat, uint32_t len, TErrorCode *err);
DXAPIDLL bool WINAPI DX_Ping (TDeviceID dvid, uint8_t id, TErrorCode *err);
DXAPIDLL bool WINAPI DX_Ping2 (TDeviceID dvid, uint32_t *num, TDxAlarmStatus *AlarmStatus, TErrorCode *err);
//DXAPIDLL bool WINAPI DX_ReadSyncData (TDeviceID dvid, uint8_t *dat, uint32_t size, uint8_t *retdat, TErrorCode *err);
DXAPIDLL bool WINAPI DX_ReadSyncData (TDeviceID dvid, const TSyncReadParam *param, uint32_t *num, uint8_t *dat, TErrorCode *err);
DXAPIDLL bool WINAPI DX_WriteSyncData (TDeviceID dvid, uint8_t *dat, uint32_t size, TErrorCode *err);
DXAPIDLL bool WINAPI DX_Reset (TDeviceID dvid, uint8_t id, TErrorCode *err);

DXAPIDLL bool WINAPI DXL_SetLED (TDeviceID dvid, uint8_t id, bool en);
DXAPIDLL bool WINAPI DXL_SetTorqueEnable (TDeviceID dvid, uint8_t id, bool en);
DXAPIDLL bool WINAPI DXL_SetTorqueEnables (TDeviceID dvid, const uint8_t *ids, const bool *ens, int num);
DXAPIDLL bool WINAPI DXL_SetTorqueEnablesEquival (TDeviceID dvid, const uint8_t *ids, int num, bool en);
DXAPIDLL bool WINAPI DXL_GetTorqueEnable (TDeviceID dvid, uint8_t id, bool *en);
DXAPIDLL bool WINAPI DXL_GetTorqueEnables (TDeviceID dvid, const uint8_t *ids, bool *en, int num);
DXAPIDLL bool WINAPI DXL_SetGoalAngle (TDeviceID dvid, uint8_t id, double angle);
DXAPIDLL bool WINAPI DXL_SetGoalAngles (TDeviceID dvid, const uint8_t *ids, const double *angles, int num);
DXAPIDLL bool WINAPI DXL_GetPresentAngle (TDeviceID dvid, uint8_t id, double *angle);
DXAPIDLL bool WINAPI DXL_GetPresentAngles (TDeviceID dvid, const uint8_t *ids, double *angles, int num);
DXAPIDLL bool WINAPI DXL_StandStillAngle (TDeviceID dvid, uint8_t id);
DXAPIDLL bool WINAPI DXL_StandStillAngles (TDeviceID dvid, const uint8_t *ids, int num);
DXAPIDLL bool WINAPI DXL_SetGoalVelocity (TDeviceID dvid, uint8_t id, double velocity);
DXAPIDLL bool WINAPI DXL_SetGoalVelocities (TDeviceID dvid, const uint8_t *ids, const double *velocities, int num);
DXAPIDLL bool WINAPI DXL_GetPresentVelocity (TDeviceID dvid, uint8_t id, double *velocity);
DXAPIDLL bool WINAPI DXL_GetPresentVelocities (TDeviceID dvid, const uint8_t *ids, double *velocities, int num);
DXAPIDLL bool WINAPI DXL_SetGoalAngleAndVelocity (TDeviceID dvid, uint8_t id, double angle, double velocity);
DXAPIDLL bool WINAPI DXL_SetGoalAnglesAndVelocities (TDeviceID dvid, const uint8_t *ids, PAngleVelocity anglevelocity, int num);
DXAPIDLL bool WINAPI DXL_SetGoalAngleAndTime (TDeviceID dvid, uint8_t id, double angle, double sec);
DXAPIDLL bool WINAPI DXL_SetGoalAnglesAndTime (TDeviceID dvid, const uint8_t *ids, const double *angles, int num, double sec);
DXAPIDLL bool WINAPI DXL_SetGoalCurrent (TDeviceID dvid, uint8_t id, double current);
DXAPIDLL bool WINAPI DXL_SetGoalCurrents (TDeviceID dvid, const uint8_t *ids, const double *currents, int num);
DXAPIDLL bool WINAPI DXL_GetPresentCurrent (TDeviceID dvid, uint8_t id, double *current);
DXAPIDLL bool WINAPI DXL_GetPresentCurrents (TDeviceID dvid, const uint8_t *ids, double *currents, int num);
DXAPIDLL bool WINAPI DXL_SetGoalPWM (TDeviceID dvid, uint8_t id, double pwm);
DXAPIDLL bool WINAPI DXL_SetGoalPWMs (TDeviceID dvid, const uint8_t *ids, const double *pwms, int num);
DXAPIDLL bool WINAPI DXL_GetPresentPWM (TDeviceID dvid, uint8_t id, double *pwm);
DXAPIDLL bool WINAPI DXL_GetPresentPWMs (TDeviceID dvid, const uint8_t *ids, double *pwms, int num);
DXAPIDLL bool WINAPI DXL_SetOperatingMode (TDeviceID dvid, uint8_t id, uint8_t mode);
DXAPIDLL bool WINAPI DXL_SetOperatingModesEquival (TDeviceID dvid, const uint8_t *ids, int num, uint8_t mode);
DXAPIDLL bool WINAPI DXL_GetOperatingMode (TDeviceID dvid, uint8_t id, uint8_t *mode);
DXAPIDLL bool WINAPI DXL_GetHWErrorCode (TDeviceID dvid, uint8_t id, uint8_t *hwerr);
DXAPIDLL TErrorCode WINAPI DXL_GetErrorCode (TDeviceID dvid, uint8_t id);
DXAPIDLL PDXL_ModelInfo WINAPI DXL_GetModelInfo (TDeviceID dvid, uint8_t id);
DXAPIDLL int WINAPI DXL_ScanDevices (TDeviceID dvid, uint8_t *ids);
DXAPIDLL bool WINAPI DXL_PrintDevicesList (int (*pf) (const char *, ...));
DXAPIDLL void WINAPI DXL_InitDevicesList (void);

#else
// When the dynamic DLL calls is used the following prototype declaration
TDeviceID (WINAPI *DX_OpenPort) (char *name, uint32_t baud) = NULL;
bool (WINAPI *DX_ClosePort) (TDeviceID dvid) = NULL;
bool (WINAPI *DX_SetBaudrate) (TDeviceID dvid, uint32_t baud) = NULL;
bool (WINAPI *DX_Active) (TDeviceID dvid) = NULL;
void (WINAPI *DX_SetTimeOutOffset) (TDeviceID dvid, uint32_t offsettime) = NULL;
double (WINAPI *GetQueryPerformanceCounter) (void) = NULL;
bool (WINAPI *DX_TxPacket) (TDeviceID dvid, uint8_t id, TInstruction inst, const uint8_t *param, uint32_t len, TErrorCode *err) = NULL;
bool (WINAPI *DX_RxPacket) (TDeviceID dvid, uint8_t *rdata, uint32_t rdatasize, uint32_t *rlen, uint32_t timeout, TErrorCode *err) = NULL;
bool (WINAPI *DX_ReadByteData) (TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *rdata, TErrorCode *err) = NULL;
bool (WINAPI *DX_WriteByteData) (TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t dat, TErrorCode *err) = NULL;
bool (WINAPI *DX_ReadWordData) (TDeviceID dvid, uint8_t id, uint16_t adr, uint16_t *rdata, TErrorCode *err) = NULL;
bool (WINAPI *DX_WriteWordData) (TDeviceID dvid, uint8_t id, uint16_t adr, uint16_t dat, TErrorCode *err) = NULL;
bool (WINAPI *DX_ReadLongData) (TDeviceID dvid, uint8_t id, uint16_t adr, uint32_t *rdata, TErrorCode *err) = NULL;
bool (WINAPI *DX_WriteLongData) (TDeviceID dvid, uint8_t id, uint16_t adr, uint32_t dat, TErrorCode *err) = NULL;
bool (WINAPI *DX_ReadBlockData) (TDeviceID dvid, uint8_t id, uint16_t adr, uint8_t *rdata, uint8_t len, TErrorCode *err) = NULL;
bool (WINAPI *DX_WriteBlockData) (TDeviceID dvid, uint8_t id, uint16_t adr, const uint8_t *dat, uint32_t len, TErrorCode *err) = NULL;
bool (WINAPI *DX_Ping) (TDeviceID dvid, uint8_t id, TErrorCode *err) = NULL;
bool (WINAPI *DX_Ping2) (TDeviceID dvid, uint32_t *num, TDxAlarmStatus *AlarmStatus, TErrorCode *err) = NULL;
//bool (WINAPI *DX_ReadSyncData) (TDeviceID dvid, uint8_t *dat, uint32_t size, uint8_t *retdat, TErrorCode *err) = NULL;
bool (WINAPI *DX_ReadSyncData) (TDeviceID dvid, const TSyncReadParam *param, uint32_t *num, uint8_t *dat, TErrorCode *err) = NULL;
bool (WINAPI *DX_WriteSyncData) (TDeviceID dvid, uint8_t *dat, uint32_t size, TErrorCode *err) = NULL;
bool (WINAPI *DX_Reset) (TDeviceID dvid, uint8_t id, TErrorCode *err) = NULL;

bool (WINAPI *DXL_SetLED) (TDeviceID dvid, uint8_t id, bool en) = NULL;
bool (WINAPI *DXL_SetTorqueEnable) (TDeviceID dvid, uint8_t id, bool en) = NULL;
bool (WINAPI *DXL_SetTorqueEnables) (TDeviceID dvid, const uint8_t *ids, const bool *ens, int num) = NULL;
bool (WINAPI *DXL_SetTorqueEnablesEquival) (TDeviceID dvid, const uint8_t *ids, int num, bool en) = NULL;
bool (WINAPI *DXL_GetTorqueEnable) (TDeviceID dvid, uint8_t id, bool *en) = NULL;
bool (WINAPI *DXL_GetTorqueEnables) (TDeviceID dvid, const uint8_t *ids, bool *en, int num) = NULL;
bool (WINAPI *DXL_SetGoalAngle) (TDeviceID dvid, uint8_t id, double angle) = NULL;
bool (WINAPI *DXL_SetGoalAngles) (TDeviceID dvid, const uint8_t *ids, const double *angles, int num) = NULL;
bool (WINAPI *DXL_GetPresentAngle) (TDeviceID dvid, uint8_t id, double *angle) = NULL;
bool (WINAPI *DXL_GetPresentAngles) (TDeviceID dvid, const uint8_t *ids, double *angles, int num) = NULL;
bool (WINAPI *DXL_StandStillAngle) (TDeviceID dvid, uint8_t id);
bool (WINAPI *DXL_StandStillAngles) (TDeviceID dvid, const uint8_t *ids, int num);
bool (WINAPI *DXL_SetGoalVelocity) (TDeviceID dvid, uint8_t id, double velocity) = NULL;
bool (WINAPI *DXL_SetGoalVelocities) (TDeviceID dvid, const uint8_t *ids, const double *velocities, int num) = NULL;
bool (WINAPI *DXL_GetPresentVelocity) (TDeviceID dvid, uint8_t id, double *velocity) = NULL;
bool (WINAPI *DXL_GetPresentVelocities) (TDeviceID dvid, const uint8_t *ids, double *velocities, int num) = NULL;
bool (WINAPI *DXL_SetGoalAngleAndVelocity) (TDeviceID dvid, uint8_t id, double angle, double velocity) = NULL;
bool (WINAPI *DXL_SetGoalAnglesAndVelocities) (TDeviceID dvid, const uint8_t *ids, PAngleVelocity anglevelocity, int num) = NULL;
bool (WINAPI *DXL_SetGoalAngleAndTime) (TDeviceID dvid, uint8_t id, double angle, double sec) = NULL;
bool (WINAPI *DXL_SetGoalAnglesAndTime) (TDeviceID dvid, const uint8_t *ids, const double *angles, int num, double sec) = NULL;
bool (WINAPI *DXL_SetGoalCurrent) (TDeviceID dvid, uint8_t id, double current) = NULL;
bool (WINAPI *DXL_SetGoalCurrents) (TDeviceID dvid, const uint8_t *ids, const double *currents, int num) = NULL;
bool (WINAPI *DXL_GetPresentCurrent) (TDeviceID dvid, uint8_t id, double *current) = NULL;
bool (WINAPI *DXL_GetPresentCurrents) (TDeviceID dvid, const uint8_t *ids, double *currents, int num) = NULL;
bool (WINAPI *DXL_SetGoalPWM) (TDeviceID dvid, uint8_t id, double pwm) = NULL;
bool (WINAPI *DXL_SetGoalPWMs) (TDeviceID dvid, const uint8_t *ids, const double *pwms, int num) = NULL;
bool (WINAPI *DXL_GetPresentPWM) (TDeviceID dvid, uint8_t id, double *pwm) = NULL;
bool (WINAPI *DXL_GetPresentPWMs) (TDeviceID dvid, const uint8_t *ids, double *pwms, int num) = NULL;
bool (WINAPI *DXL_SetOperatingMode) (TDeviceID dvid, uint8_t id, uint8_t mode) = NULL;
bool (WINAPI *DXL_SetOperatingModesEquival) (TDeviceID dvid, const uint8_t *ids, int num, uint8_t mode) = NULL;
bool (WINAPI *DXL_GetOperatingMode) (TDeviceID dvid, uint8_t id, uint8_t *mode) = NULL;
bool (WINAPI *DXL_GetHWErrorCode) (TDeviceID dvid, uint8_t id, uint8_t *hwerr) = NULL;
TErrorCode (WINAPI *DXL_GetErrorCode) (TDeviceID dvid, uint8_t id) = NULL;
PDXL_ModelInfo (WINAPI *DXL_GetModelInfo) (TDeviceID dvid, uint8_t id) = NULL;
int (WINAPI *DXL_ScanDevices) (TDeviceID dvid, uint8_t *ids) = NULL;
bool (WINAPI *DXL_PrintDevicesList) (int (*pf) (const char *, ...)) = NULL;
void (WINAPI *DXL_InitDevicesList) (void) = NULL;

static HMODULE hModule = NULL;

#ifdef _WIN32
#ifdef _WIN64
#define DLLNAME   "dxlib_x64.dll"
#else
#define DLLNAME   "dxlib_x32.dll"
#endif
#else
#ifdef __x86_64__
#define DLLNAME   "dxlib_x64.dll"
#else
#define DLLNAME   "dxlib_x32.dll"
#endif
#endif

// Load the DLL
bool LoadDLL (void) {
  hModule = LoadLibrary (DLLNAME);
  if (hModule == NULL) {
    return false;
  }

  DX_OpenPort                = (void *)GetProcAddress (hModule, "DX_OpenPort");
  DX_ClosePort               = (void *)GetProcAddress (hModule, "DX_ClosePort");
  DX_SetBaudrate             = (void *)GetProcAddress (hModule, "DX_SetBaudrate");
  DX_Active                  = (void *)GetProcAddress (hModule, "DX_Active");
  DX_SetTimeOutOffset        = (void *)GetProcAddress (hModule, "DX_SetTimeOutOffset");
  GetQueryPerformanceCounter = (void *)GetProcAddress (hModule, "GetQueryPerformanceCounter");
  DX_TxPacket                = (void *)GetProcAddress (hModule, "DX_TxPacket");
  DX_RxPacket                = (void *)GetProcAddress (hModule, "DX_RxPacket");
  DX_ReadByteData            = (void *)GetProcAddress (hModule, "DX_ReadByteData");
  DX_WriteByteData           = (void *)GetProcAddress (hModule, "DX_WriteByteData");
  DX_ReadWordData            = (void *)GetProcAddress (hModule, "DX_ReadWordData");
  DX_WriteWordData           = (void *)GetProcAddress (hModule, "DX_WriteWordData");
  DX_ReadLongData            = (void *)GetProcAddress (hModule, "DX_ReadLongData");
  DX_WriteLongData           = (void *)GetProcAddress (hModule, "DX_WriteLongData");
  DX_ReadBlockData           = (void *)GetProcAddress (hModule, "DX_ReadBlockData");
  DX_WriteBlockData          = (void *)GetProcAddress (hModule, "DX_WriteBlockData");
  DX_ReadSyncData            = (void *)GetProcAddress (hModule, "DX_ReadSyncData");
  DX_WriteSyncData           = (void *)GetProcAddress (hModule, "DX_WriteSyncData");
  DX_Ping                    = (void *)GetProcAddress (hModule, "DX_Ping");
  DX_Ping2                   = (void *)GetProcAddress (hModule, "DX_Ping2");
  DX_Reset                   = (void *)GetProcAddress (hModule, "DX_Reset");

  DXL_SetTorqueEnable           = (void *)GetProcAddress (hModule, "DXL_SetTorqueEnable");
  DXL_SetTorqueEnables          = (void *)GetProcAddress (hModule, "DXL_SetTorqueEnables");
  DXL_SetTorqueEnablesEquival   = (void *)GetProcAddress (hModule, "DXL_SetTorqueEnablesEquival");
  DXL_GetTorqueEnable           = (void *)GetProcAddress (hModule, "DXL_GetTorqueEnable");
  DXL_GetTorqueEnables          = (void *)GetProcAddress (hModule, "DXL_GetTorqueEnables");
  DXL_SetLED                    = (void *)GetProcAddress (hModule, "DXL_SetLED");
  DXL_SetGoalAngle              = (void *)GetProcAddress (hModule, "DXL_SetGoalAngle");
  DXL_SetGoalAngles             = (void *)GetProcAddress (hModule, "DXL_SetGoalAngles");
  DXL_GetPresentAngle           = (void *)GetProcAddress (hModule, "DXL_GetPresentAngle");
  DXL_GetPresentAngles          = (void *)GetProcAddress (hModule, "DXL_GetPresentAngles");
  DXL_StandStillAngle           = (void *)GetProcAddress (hModule, "DXL_StandStillAngle");
  DXL_StandStillAngles          = (void *)GetProcAddress (hModule, "DXL_StandStillAngles");
  DXL_SetGoalVelocity           = (void *)GetProcAddress (hModule, "DXL_SetGoalVelocity");
  DXL_SetGoalVelocities         = (void *)GetProcAddress (hModule, "DXL_SetGoalVelocities");
  DXL_GetPresentVelocity        = (void *)GetProcAddress (hModule, "DXL_GetPresentVelocity");
  DXL_GetPresentVelocities      = (void *)GetProcAddress (hModule, "DXL_GetPresentVelocities");
  DXL_SetGoalAngleAndVelocity   = (void *)GetProcAddress (hModule, "DXL_SetGoalAngleAndVelocity");
  DXL_SetGoalAnglesAndVelocities= (void *)GetProcAddress (hModule, "DXL_SetGoalAnglesAndVelocities");
  DXL_SetGoalAngleAndTime       = (void *)GetProcAddress (hModule, "DXL_SetGoalAngleAndTime");
  DXL_SetGoalAnglesAndTime      = (void *)GetProcAddress (hModule, "DXL_SetGoalAnglesAndTime");
  DXL_SetGoalPWM                = (void *)GetProcAddress (hModule, "DXL_SetGoalPWM");
  DXL_SetGoalPWMs               = (void *)GetProcAddress (hModule, "DXL_SetGoalPWMs");
  DXL_GetPresentPWM             = (void *)GetProcAddress (hModule, "DXL_GetPresentPWM");
  DXL_GetPresentPWMs            = (void *)GetProcAddress (hModule, "DXL_GetPresentPWMs");
  DXL_SetGoalCurrent            = (void *)GetProcAddress (hModule, "DXL_SetGoalCurrent");
  DXL_SetGoalCurrents           = (void *)GetProcAddress (hModule, "DXL_SetGoalCurrents");
  DXL_GetPresentCurrent         = (void *)GetProcAddress (hModule, "DXL_GetPresentCurrent");
  DXL_GetPresentCurrents        = (void *)GetProcAddress (hModule, "DXL_GetPresentCurrents");
  DXL_SetOperatingMode          = (void *)GetProcAddress (hModule, "DXL_SetOperatingMode");
  DXL_SetOperatingModesEquival  = (void *)GetProcAddress (hModule, "DXL_SetOperatingModesEquival");
  DXL_GetOperatingMode          = (void *)GetProcAddress (hModule, "DXL_GetOperatingMode");
  DXL_GetHWErrorCode            = (void *)GetProcAddress (hModule, "DXL_GetHWErrorCode");
  DXL_GetErrorCode              = (void *)GetProcAddress (hModule, "DXL_GetErrorCode");
  DXL_GetModelInfo              = (void *)GetProcAddress (hModule, "DXL_GetModelInfo");
  DXL_ScanDevices               = (void *)GetProcAddress (hModule, "DXL_ScanDevices");
  DXL_PrintDevicesList          = (void *)GetProcAddress (hModule, "DXL_PrintDevicesList");
  DXL_InitDevicesList           = (void *)GetProcAddress (hModule, "DXL_InitDevicesList");

  return
    (DX_OpenPort                != NULL) &&
    (DX_ClosePort               != NULL) &&
    (DX_SetBaudrate             != NULL) &&
    (DX_Active                  != NULL) &&
    (DX_SetTimeOutOffset        != NULL) &&
    (GetQueryPerformanceCounter != NULL) &&
    (DX_TxPacket                != NULL) &&
    (DX_RxPacket                != NULL) &&
    (DX_ReadByteData            != NULL) &&
    (DX_WriteByteData           != NULL) &&
    (DX_ReadWordData            != NULL) &&
    (DX_WriteWordData           != NULL) &&
    (DX_ReadLongData            != NULL) &&
    (DX_WriteLongData           != NULL) &&
    (DX_ReadBlockData           != NULL) &&
    (DX_WriteBlockData          != NULL) &&
    (DX_ReadSyncData            != NULL) &&
    (DX_WriteSyncData           != NULL) &&
    (DX_Ping                    != NULL) &&
    (DX_Ping2                   != NULL) &&
    (DX_Reset                   != NULL) &&

    (DXL_SetTorqueEnable           != NULL) &&
    (DXL_SetTorqueEnables          != NULL) &&
    (DXL_SetTorqueEnablesEquival   != NULL) &&
    (DXL_GetTorqueEnable           != NULL) &&
    (DXL_GetTorqueEnables          != NULL) &&
    (DXL_SetLED                    != NULL) &&
    (DXL_SetGoalAngle              != NULL) &&
    (DXL_SetGoalAngles             != NULL) &&
    (DXL_GetPresentAngle           != NULL) &&
    (DXL_GetPresentAngles          != NULL) &&
    (DXL_StandStillAngle           != NULL) &&
    (DXL_StandStillAngles          != NULL) &&
    (DXL_SetGoalVelocity           != NULL) &&
    (DXL_SetGoalVelocities         != NULL) &&
    (DXL_GetPresentVelocity        != NULL) &&
    (DXL_GetPresentVelocities      != NULL) &&
    (DXL_SetGoalAngleAndVelocity   != NULL) &&
    (DXL_SetGoalAnglesAndVelocities!= NULL) &&
    (DXL_SetGoalAngleAndTime       != NULL) &&
    (DXL_SetGoalAnglesAndTime      != NULL) &&
    (DXL_SetGoalCurrent            != NULL) &&
    (DXL_SetGoalCurrents           != NULL) &&
    (DXL_GetPresentCurrent         != NULL) &&
    (DXL_GetPresentCurrents        != NULL) &&
    (DXL_SetGoalPWM                != NULL) &&
    (DXL_SetGoalPWMs               != NULL) &&
    (DXL_GetPresentPWM             != NULL) &&
    (DXL_GetPresentPWMs            != NULL) &&
    (DXL_SetOperatingMode          != NULL) &&
    (DXL_SetOperatingModesEquival  != NULL) &&
    (DXL_GetOperatingMode          != NULL) &&
    (DXL_GetHWErrorCode            != NULL) &&
    (DXL_GetErrorCode              != NULL) &&
    (DXL_GetModelInfo              != NULL) &&
    (DXL_ScanDevices               != NULL) &&
    (DXL_PrintDevicesList          != NULL) &&
    (DXL_InitDevicesList           != NULL);
}

// Unload the DLL
void UnloadDLL (void) {
  if (hModule != NULL) {
    FreeLibrary (hModule);
    DX_OpenPort                = NULL;
    DX_ClosePort               = NULL;
    DX_SetBaudrate             = NULL;
    DX_Active                  = NULL;
    DX_SetTimeOutOffset        = NULL;
    GetQueryPerformanceCounter = NULL;
    DX_TxPacket                = NULL;
    DX_RxPacket                = NULL;
    DX_ReadByteData            = NULL;
    DX_WriteByteData           = NULL;
    DX_ReadWordData            = NULL;
    DX_WriteWordData           = NULL;
    DX_ReadLongData            = NULL;
    DX_WriteLongData           = NULL;
    DX_ReadBlockData           = NULL;
    DX_WriteBlockData          = NULL;
    DX_ReadSyncData            = NULL;
    DX_WriteSyncData           = NULL;
    DX_Ping                    = NULL;
    DX_Ping2                   = NULL;
    DX_Reset                   = NULL;

    DXL_SetTorqueEnable           = NULL;
    DXL_SetTorqueEnables          = NULL;
    DXL_SetTorqueEnablesEquival   = NULL;
    DXL_GetTorqueEnable           = NULL;
    DXL_GetTorqueEnables          = NULL;
    DXL_SetLED                    = NULL;
    DXL_SetGoalAngle              = NULL;
    DXL_SetGoalAngles             = NULL;
    DXL_GetPresentAngle           = NULL;
    DXL_GetPresentAngles          = NULL;
    DXL_StandStillAngle           = NULL;
    DXL_StandStillAngles          = NULL;
    DXL_SetGoalVelocity           = NULL;
    DXL_SetGoalVelocities         = NULL;
    DXL_GetPresentVelocity        = NULL;
    DXL_GetPresentVelocities      = NULL;
    DXL_SetGoalAngleAndVelocity   = NULL;
    DXL_SetGoalAnglesAndVelocities= NULL;
    DXL_SetGoalAngleAndTime       = NULL;
    DXL_SetGoalAnglesAndTime      = NULL;
    DXL_SetGoalCurrent            = NULL;
    DXL_SetGoalCurrents           = NULL;
    DXL_GetPresentCurrent         = NULL;
    DXL_GetPresentCurrents        = NULL;
    DXL_SetGoalPWM                = NULL;
    DXL_SetGoalPWMs               = NULL;
    DXL_GetPresentPWM             = NULL;
    DXL_GetPresentPWMs            = NULL;
    DXL_SetOperatingMode          = NULL;
    DXL_SetOperatingModesEquival  = NULL;
    DXL_GetOperatingMode          = NULL;
    DXL_GetHWErrorCode            = NULL;
    DXL_GetErrorCode              = NULL;
    DXL_GetModelInfo              = NULL;
    DXL_ScanDevices               = NULL;
    DXL_PrintDevicesList          = NULL;
    DXL_InitDevicesList           = NULL;
  }
}

#endif

#ifdef __cplusplus
}
#endif

#endif //_DXLIB_H_INCLUDE
